"""UiPath core exceptions module.

This module exposes common UiPath exceptions.
"""

from .errors import (
    ErrorCategory,
    UiPathFaultedTriggerError,
    UiPathPendingTriggerError,
)

__all__ = ["UiPathFaultedTriggerError", "UiPathPendingTriggerError", "ErrorCategory"]



"""Module defining resume trigger types and data models."""

from enum import Enum
from typing import Any

from pydantic import BaseModel, ConfigDict, Field


class UiPathResumeTriggerType(str, Enum):
    """Constants representing different types of resume job triggers in the system."""

    NONE = "None"
    QUEUE_ITEM = "QueueItem"
    JOB = "Job"
    TASK = "Task"
    TIMER = "Timer"
    INBOX = "Inbox"
    API = "Api"
    DEEP_RAG = "DeepRag"
    BATCH_RAG = "BatchRag"
    INDEX_INGESTION = "IndexIngestion"
    IXP_EXTRACTION = "IxpExtraction"
    IXP_VS_ESCALATION = "IxpVsEscalation"


class UiPathResumeTriggerName(str, Enum):
    """Constants representing specific names for resume job triggers in the system."""

    UNKNOWN = "Unknown"
    QUEUE_ITEM = "QueueItem"
    JOB = "Job"
    TASK = "Task"
    ESCALATION = "Escalation"
    TIMER = "Timer"
    INBOX = "Inbox"
    API = "Api"
    DEEP_RAG = "DeepRag"
    BATCH_RAG = "BatchRag"
    INDEX_INGESTION = "IndexIngestion"
    EXTRACTION = "Extraction"
    IXP_VS_ESCALATION = "IxpVsEscalation"
    JOB_RAW = "JobRaw"
    INDEX_INGESTION_RAW = "IndexIngestionRaw"
    DEEP_RAG_RAW = "DeepRagRaw"


class UiPathApiTrigger(BaseModel):
    """API resume trigger request."""

    inbox_id: str | None = Field(default=None, alias="inboxId")
    request: Any = None

    model_config = ConfigDict(validate_by_name=True)


class UiPathResumeTrigger(BaseModel):
    """Information needed to resume execution."""

    interrupt_id: str | None = Field(default=None, alias="interruptId")
    trigger_type: UiPathResumeTriggerType = Field(
        default=UiPathResumeTriggerType.API, alias="triggerType"
    )
    trigger_name: UiPathResumeTriggerName = Field(
        default=UiPathResumeTriggerName.UNKNOWN, alias="triggerName", exclude=True
    )
    item_key: str | None = Field(default=None, alias="itemKey")
    api_resume: UiPathApiTrigger | None = Field(default=None, alias="apiResume")
    folder_path: str | None = Field(default=None, alias="folderPath")
    folder_key: str | None = Field(default=None, alias="folderKey")
    payload: Any | None = Field(default=None, alias="interruptObject", exclude=True)

    model_config = ConfigDict(validate_by_name=True)



"""UiPath Feature Flags.

Local-only feature flag registry for the UiPath SDK.
"""

from .feature_flags import FeatureFlags, FeatureFlagsManager

__all__ = [
    "FeatureFlags",
    "FeatureFlagsManager",
]



"""Custom span processors for UiPath execution tracing."""

from typing import cast

from opentelemetry import context as context_api
from opentelemetry import trace
from opentelemetry.sdk.trace import ReadableSpan, Span, SpanProcessor
from opentelemetry.sdk.trace.export import (
    BatchSpanProcessor,
    SimpleSpanProcessor,
    SpanExporter,
)

from uipath.core.tracing.types import UiPathTraceSettings


class UiPathExecutionTraceProcessorMixin:
    """Mixin that propagates execution.id and optionally filters spans."""

    _settings: UiPathTraceSettings | None = None

    def on_start(self, span: Span, parent_context: context_api.Context | None = None):
        """Called when a span is started."""
        parent_span: Span | None
        if parent_context:
            parent_span = cast(Span, trace.get_current_span(parent_context))
        else:
            parent_span = cast(Span, trace.get_current_span())

        if parent_span and parent_span.is_recording() and parent_span.attributes:
            execution_id = parent_span.attributes.get("execution.id")
            if execution_id:
                span.set_attribute("execution.id", execution_id)

    def on_end(self, span: ReadableSpan):
        """Called when a span ends. Filters before delegating to parent."""
        span_filter = self._settings.span_filter if self._settings else None
        if span_filter is None or span_filter(span):
            parent = cast(SpanProcessor, super())
            parent.on_end(span)


class UiPathExecutionBatchTraceProcessor(
    UiPathExecutionTraceProcessorMixin, BatchSpanProcessor
):
    """Batch span processor that propagates execution.id and optionally filters."""

    def __init__(
        self,
        span_exporter: SpanExporter,
        settings: UiPathTraceSettings | None = None,
    ):
        """Initialize the batch trace processor."""
        super().__init__(span_exporter)
        self._settings = settings


class UiPathExecutionSimpleTraceProcessor(
    UiPathExecutionTraceProcessorMixin, SimpleSpanProcessor
):
    """Simple span processor that propagates execution.id and optionally filters."""

    def __init__(
        self,
        span_exporter: SpanExporter,
        settings: UiPathTraceSettings | None = None,
    ):
        """Initialize the simple trace processor."""
        super().__init__(span_exporter)
        self._settings = settings


__all__ = [
    "UiPathExecutionBatchTraceProcessor",
    "UiPathExecutionSimpleTraceProcessor",
]



"""Guardrails models for UiPath Platform."""

from enum import Enum
from typing import Annotated, Any, Callable, Literal

from pydantic import BaseModel, ConfigDict, Field, field_validator


def _decapitalize_first_letter(s: str) -> str:
    """Convert first letter to lowercase (e.g., 'SimpleText' -> 'simpleText')."""
    if not s or len(s) == 0:
        return s
    if len(s) == 1:
        return s.lower()
    return s[0].lower() + s[1:]


class GuardrailValidationResultType(str, Enum):
    """Guardrail validation result type enumeration."""

    PASSED = "passed"
    VALIDATION_FAILED = "validation_failed"
    ENTITLEMENTS_MISSING = "entitlements_missing"
    FEATURE_DISABLED = "feature_disabled"


class GuardrailValidationResult(BaseModel):
    """Result returned from validating input with a given guardrail.

    Attributes:
        result: The validation result type.
        reason: Textual explanation describing why the validation passed or failed.
    """

    model_config = ConfigDict(populate_by_name=True)

    result: GuardrailValidationResultType = Field(
        alias="result", description="Validation result."
    )
    reason: str = Field(
        alias="reason", description="Explanation for the validation result."
    )


class FieldSource(str, Enum):
    """Field source enumeration."""

    INPUT = "input"
    OUTPUT = "output"


class ApplyTo(str, Enum):
    """Apply to enumeration."""

    INPUT = "input"
    INPUT_AND_OUTPUT = "inputAndOutput"
    OUTPUT = "output"


class FieldReference(BaseModel):
    """Field reference model."""

    path: str
    source: FieldSource

    model_config = ConfigDict(populate_by_name=True, extra="allow")

    @field_validator("source", mode="before")
    @classmethod
    def normalize_type(cls, v: Any) -> Any:
        """Normalize type by decapitalizing first letter."""
        return _decapitalize_first_letter(v) if isinstance(v, str) else v


class SelectorType(str, Enum):
    """Selector type enumeration."""

    ALL = "all"
    SPECIFIC = "specific"


class AllFieldsSelector(BaseModel):
    """All fields selector."""

    selector_type: Literal["all"] = Field(alias="$selectorType")
    sources: list[FieldSource]

    model_config = ConfigDict(populate_by_name=True, extra="allow")

    @field_validator("sources", mode="before")
    @classmethod
    def normalize_sources(cls, v: Any) -> Any:
        """Normalize sources by decapitalizing first letter of each item."""
        if isinstance(v, list):
            return [
                _decapitalize_first_letter(item) if isinstance(item, str) else item
                for item in v
            ]
        return v


class SpecificFieldsSelector(BaseModel):
    """Specific fields selector."""

    selector_type: Literal["specific"] = Field(alias="$selectorType")
    fields: list[FieldReference]

    model_config = ConfigDict(populate_by_name=True, extra="allow")


FieldSelector = Annotated[
    AllFieldsSelector | SpecificFieldsSelector,
    Field(discriminator="selector_type"),
]


class RuleType(str, Enum):
    """Rule type enumeration."""

    BOOLEAN = "boolean"
    NUMBER = "number"
    UNIVERSAL = "always"
    WORD = "word"


class WordRule(BaseModel):
    """Word rule model."""

    rule_type: Literal["word"] = Field(alias="$ruleType")
    field_selector: FieldSelector = Field(alias="fieldSelector")
    rule_description: str | None = Field(
        default=None,
        exclude=True,
        description="Human-friendly description of the rule condition.",
    )
    detects_violation: Callable[[str], bool] = Field(
        exclude=True,
        description="Function that returns True if the string violates the rule (validation should fail).",
    )

    model_config = ConfigDict(populate_by_name=True, extra="allow")


class UniversalRule(BaseModel):
    """Universal rule model."""

    rule_type: Literal["always"] = Field(alias="$ruleType")
    apply_to: ApplyTo = Field(alias="applyTo")
    rule_description: str | None = Field(
        default=None,
        exclude=True,
        description="Human-friendly description of the rule condition.",
    )

    model_config = ConfigDict(populate_by_name=True, extra="allow")

    @field_validator("apply_to", mode="before")
    @classmethod
    def normalize_type(cls, v: Any) -> Any:
        """Normalize type by decapitalizing first letter."""
        return _decapitalize_first_letter(v) if isinstance(v, str) else v


class NumberRule(BaseModel):
    """Number rule model."""

    rule_type: Literal["number"] = Field(alias="$ruleType")
    field_selector: FieldSelector = Field(alias="fieldSelector")
    rule_description: str | None = Field(
        default=None,
        exclude=True,
        description="Human-friendly description of the rule condition.",
    )
    detects_violation: Callable[[float], bool] = Field(
        exclude=True,
        description="Function that returns True if the number violates the rule (validation should fail).",
    )

    model_config = ConfigDict(populate_by_name=True, extra="allow")


class BooleanRule(BaseModel):
    """Boolean rule model."""

    rule_type: Literal["boolean"] = Field(alias="$ruleType")
    field_selector: FieldSelector = Field(alias="fieldSelector")
    rule_description: str | None = Field(
        default=None,
        exclude=True,
        description="Human-friendly description of the rule condition.",
    )
    detects_violation: Callable[[bool], bool] = Field(
        exclude=True,
        description="Function that returns True if the boolean violates the rule (validation should fail).",
    )

    model_config = ConfigDict(populate_by_name=True, extra="allow")


Rule = Annotated[
    WordRule | NumberRule | BooleanRule | UniversalRule,
    Field(discriminator="rule_type"),
]


class GuardrailScope(str, Enum):
    """Guardrail scope enumeration."""

    AGENT = "Agent"
    LLM = "Llm"
    TOOL = "Tool"


class GuardrailSelector(BaseModel):
    """Guardrail selector model."""

    scopes: list[GuardrailScope] = Field(default=[GuardrailScope.TOOL])
    match_names: list[str] | None = Field(None, alias="matchNames")

    model_config = ConfigDict(populate_by_name=True, extra="allow")


class BaseGuardrail(BaseModel):
    """Base guardrail model."""

    id: str
    name: str
    description: str | None = None
    enabled_for_evals: bool = Field(True, alias="enabledForEvals")
    selector: GuardrailSelector

    model_config = ConfigDict(populate_by_name=True, extra="allow")


class DeterministicGuardrail(BaseGuardrail):
    """Deterministic guardrail model."""

    guardrail_type: Literal["custom"] = Field(alias="$guardrailType")
    rules: list[Rule]

    model_config = ConfigDict(populate_by_name=True, extra="allow")



import json

from tests.conftest import SpanCapture


def test_traced_with_pydantic_model_input(span_capture: SpanCapture):
    """Test tracing with Pydantic model as input."""
    from pydantic import BaseModel

    from uipath.core.tracing.decorators import traced

    class UserModel(BaseModel):
        name: str
        age: int

    @traced(name="process_user")
    def process_user(user: UserModel):
        return f"{user.name} is {user.age}"

    user = UserModel(name="Alice", age=30)
    result = process_user(user)

    assert result == "Alice is 30"

    spans = span_capture.get_spans()
    assert len(spans) == 1

    span = spans[0]
    # Verify input was serialized
    input_value = span.attributes.get("input.value")
    assert input_value is not None
    input_data = json.loads(input_value)
    assert input_data["user"]["name"] == "Alice"
    assert input_data["user"]["age"] == 30


def test_traced_with_dataclass_input(span_capture: SpanCapture):
    """Test tracing with dataclass as input."""
    from dataclasses import dataclass

    from uipath.core.tracing.decorators import traced

    @dataclass
    class Product:
        name: str
        price: float

    @traced(name="calculate_total")
    def calculate_total(product: Product, quantity: int):
        return product.price * quantity

    product = Product(name="Widget", price=9.99)
    result = calculate_total(product, 5)

    assert result == 49.95

    spans = span_capture.get_spans()
    assert len(spans) == 1

    span = spans[0]
    input_value = span.attributes.get("input.value")
    assert input_value is not None
    input_data = json.loads(input_value)
    assert input_data["product"]["name"] == "Widget"
    assert input_data["product"]["price"] == 9.99


def test_traced_with_enum_input(span_capture: SpanCapture):
    """Test tracing with enum as input."""
    from enum import Enum

    from uipath.core.tracing.decorators import traced

    class Status(Enum):
        PENDING = "pending"
        COMPLETED = "completed"

    @traced(name="update_status")
    def update_status(status: Status):
        return f"Status is {status.value}"

    result = update_status(Status.COMPLETED)

    assert result == "Status is completed"

    spans = span_capture.get_spans()
    assert len(spans) == 1

    span = spans[0]
    input_value = span.attributes.get("input.value")
    assert input_value is not None
    input_data = json.loads(input_value)
    assert input_data["status"] == "completed"


def test_traced_with_datetime_input(span_capture: SpanCapture):
    """Test tracing with datetime as input."""
    from datetime import datetime

    from uipath.core.tracing.decorators import traced

    @traced(name="process_timestamp")
    def process_timestamp(timestamp: datetime):
        return timestamp.isoformat()

    dt = datetime(2024, 1, 15, 10, 30, 0)
    process_timestamp(dt)

    spans = span_capture.get_spans()
    assert len(spans) == 1

    span = spans[0]
    input_value = span.attributes.get("input.value")
    assert input_value is not None
    input_data = json.loads(input_value)
    assert "2024-01-15" in input_data["timestamp"]


def test_traced_with_complex_return_value(span_capture: SpanCapture):
    """Test tracing with complex return value."""
    from typing import Any

    from pydantic import BaseModel

    from uipath.core.tracing.decorators import traced

    class Result(BaseModel):
        success: bool
        data: dict[str, Any]

    @traced(name="get_result")
    def get_result():
        return Result(success=True, data={"key": "value"})

    get_result()

    spans = span_capture.get_spans()
    assert len(spans) == 1

    span = spans[0]
    output_value = span.attributes.get("output.value")
    assert output_value is not None
    output_data = json.loads(output_value)
    assert output_data["success"] is True
    assert output_data["data"]["key"] == "value"


def test_traced_with_set_and_tuple(span_capture: SpanCapture):
    """Test tracing with set and tuple inputs."""
    from typing import Set, Tuple

    from uipath.core.tracing.decorators import traced

    @traced(name="process_collections")
    def process_collections(items: Set[int], pair: Tuple[int, ...]) -> int:
        return len(items) + len(pair)

    result = process_collections({1, 2, 3}, (4, 5))

    assert result == 5

    spans = span_capture.get_spans()
    assert len(spans) == 1

    span = spans[0]
    input_value = span.attributes.get("input.value")
    assert input_value is not None
    input_data = json.loads(input_value)
    # Sets and tuples should be converted to lists
    assert isinstance(input_data["items"], list)
    assert isinstance(input_data["pair"], list)



"""Citation events for message content."""

from __future__ import annotations

from pydantic import BaseModel, ConfigDict, Field

from .error import UiPathConversationErrorEvent


class UiPathConversationCitationStartEvent(BaseModel):
    """Indicates the start of a citation target in a content part."""

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class UiPathConversationCitationEndEvent(BaseModel):
    """Indicates the end of a citation target in a content part."""

    sources: list[UiPathConversationCitationSource]

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class UiPathConversationCitationEvent(BaseModel):
    """Encapsulates sub-events related to citations."""

    citation_id: str = Field(..., alias="citationId")
    start: UiPathConversationCitationStartEvent | None = Field(
        None, alias="startCitation"
    )
    end: UiPathConversationCitationEndEvent | None = Field(None, alias="endCitation")
    error: UiPathConversationErrorEvent | None = Field(None, alias="citationError")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class UiPathConversationCitationSourceBase(BaseModel):
    """Represents a citation source with common base fields."""

    title: str
    number: int

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class UiPathConversationCitationSourceUrl(UiPathConversationCitationSourceBase):
    """Represents a citation source that can be rendered as a link (URL)."""

    url: str

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class UiPathConversationCitationSourceMedia(UiPathConversationCitationSourceBase):
    """Represents a citation source that references media, such as a PDF document."""

    mime_type: str | None = Field(..., alias="mimeType")
    download_url: str | None = Field(None, alias="downloadUrl")
    page_number: str | None = Field(None, alias="pageNumber")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


UiPathConversationCitationSource = (
    UiPathConversationCitationSourceUrl | UiPathConversationCitationSourceMedia
)


class UiPathConversationCitationData(BaseModel):
    """Represents the core data of a citation or reference inside a content part."""

    offset: int
    length: int
    sources: list[UiPathConversationCitationSource]

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)


class UiPathConversationCitation(UiPathConversationCitationData):
    """Represents a citation or reference inside a content part."""

    citation_id: str = Field(..., alias="citationId")
    created_at: str = Field(..., alias="createdAt")
    updated_at: str = Field(..., alias="updatedAt")

    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)



from typing import Any

from pydantic import BaseModel

from ..tracing.decorators import traced
from ._evaluators import (
    evaluate_boolean_rule,
    evaluate_number_rule,
    evaluate_universal_rule,
    evaluate_word_rule,
)
from .guardrails import (
    AllFieldsSelector,
    ApplyTo,
    BooleanRule,
    DeterministicGuardrail,
    FieldSource,
    GuardrailValidationResult,
    GuardrailValidationResultType,
    NumberRule,
    SpecificFieldsSelector,
    UniversalRule,
    WordRule,
)


class DeterministicGuardrailsService(BaseModel):
    @traced("evaluate_pre_deterministic_guardrail", run_type="uipath")
    def evaluate_pre_deterministic_guardrail(
        self,
        input_data: dict[str, Any],
        guardrail: DeterministicGuardrail,
    ) -> GuardrailValidationResult:
        """Evaluate deterministic guardrail rules against input data (pre-execution)."""
        # Check if guardrail contains any output-dependent rules
        has_output_rule = self._has_output_dependent_rule(guardrail, [ApplyTo.OUTPUT])

        # If guardrail has output-dependent rules, skip evaluation in pre-execution
        # Output rules will be evaluated during post-execution
        if has_output_rule:
            return GuardrailValidationResult(
                result=GuardrailValidationResultType.PASSED,
                reason="No rules to apply for input data.",
            )
        return self._evaluate_deterministic_guardrail(
            input_data=input_data,
            output_data={},
            guardrail=guardrail,
        )

    @traced("evaluate_post_deterministic_guardrails", run_type="uipath")
    def evaluate_post_deterministic_guardrail(
        self,
        input_data: dict[str, Any],
        output_data: dict[str, Any],
        guardrail: DeterministicGuardrail,
    ) -> GuardrailValidationResult:
        """Evaluate deterministic guardrail rules against input and output data."""
        # Check if guardrail contains any output-dependent rules
        has_output_rule = self._has_output_dependent_rule(
            guardrail, [ApplyTo.OUTPUT, ApplyTo.INPUT_AND_OUTPUT]
        )

        # If guardrail has no output-dependent rules, skip post-execution evaluation
        # Only input rules exist and they should have been evaluated during pre-execution
        if not has_output_rule:
            return GuardrailValidationResult(
                result=GuardrailValidationResultType.PASSED,
                reason="No rules to apply for output data.",
            )

        return self._evaluate_deterministic_guardrail(
            input_data=input_data,
            output_data=output_data,
            guardrail=guardrail,
        )

    @staticmethod
    def _has_output_dependent_rule(
        guardrail: DeterministicGuardrail,
        universal_rules_apply_to_values: list[ApplyTo],
    ) -> bool:
        """Check if at least one rule EXCLUSIVELY requires output data.

        Args:
            guardrail: The guardrail to check
            universal_rules_apply_to_values: List of ApplyTo values to consider as output-dependent for UniversalRules.

        Returns:
            True if at least one rule exclusively depends on output data, False otherwise.
        """
        for rule in guardrail.rules:
            # UniversalRule: only return True if it applies to values in universal_rules_apply_to_values
            if isinstance(rule, UniversalRule):
                if rule.apply_to in universal_rules_apply_to_values:
                    return True
            # Rules with field_selector
            elif isinstance(rule, (WordRule, NumberRule, BooleanRule)):
                field_selector = rule.field_selector
                # AllFieldsSelector applies to both input and output, not exclusively output
                # SpecificFieldsSelector: only return True if at least one field has OUTPUT source
                if isinstance(field_selector, SpecificFieldsSelector):
                    if field_selector.fields and any(
                        field.source == FieldSource.OUTPUT
                        for field in field_selector.fields
                    ):
                        return True
                elif isinstance(field_selector, AllFieldsSelector):
                    if FieldSource.OUTPUT in field_selector.sources:
                        return True

        return False

    @staticmethod
    def _evaluate_deterministic_guardrail(
        input_data: dict[str, Any],
        output_data: dict[str, Any],
        guardrail: DeterministicGuardrail,
    ) -> GuardrailValidationResult:
        """Evaluate deterministic guardrail rules against input and output data.

        Validation fails only if ALL guardrail rules are violated.
        """
        validated_conditions: list[str] = []

        for rule in guardrail.rules:
            if isinstance(rule, WordRule):
                passed, reason = evaluate_word_rule(rule, input_data, output_data)
            elif isinstance(rule, NumberRule):
                passed, reason = evaluate_number_rule(rule, input_data, output_data)
            elif isinstance(rule, BooleanRule):
                passed, reason = evaluate_boolean_rule(rule, input_data, output_data)
            elif isinstance(rule, UniversalRule):
                passed, reason = evaluate_universal_rule(rule, output_data)
            else:
                return GuardrailValidationResult(
                    result=GuardrailValidationResultType.VALIDATION_FAILED,
                    reason=f"Unknown rule type: {type(rule)}",
                )
            validated_conditions.append(reason)
            if passed:
                return GuardrailValidationResult(
                    result=GuardrailValidationResultType.PASSED,
                    reason=reason,
                )

        has_always_rule = any(
            condition == "Always rule enforced" for condition in validated_conditions
        )

        validated_conditions_str = ", ".join(validated_conditions)
        final_reason = (
            "Always rule enforced"
            if has_always_rule
            else f"Data matched all guardrail conditions: [{validated_conditions_str}]"
        )

        return GuardrailValidationResult(
            result=GuardrailValidationResultType.VALIDATION_FAILED,
            reason=final_reason,
        )



"""Tracing types for UiPath SDK."""

from typing import Callable

from opentelemetry.sdk.trace import ReadableSpan
from pydantic import BaseModel, Field


class UiPathTraceSettings(BaseModel):
    """Trace settings for UiPath SDK."""

    model_config = {"arbitrary_types_allowed": True}  # Needed for Callable

    span_filter: Callable[[ReadableSpan], bool] | None = Field(
        default=None,
        description=(
            "Optional filter to decide whether a span should be exported. "
            "Called when a span ends with a ReadableSpan argument. "
            "Return True to export, False to skip."
        ),
    )



"""Module containing UiPath trigger definitions."""

__all__ = [
    "UiPathResumeTrigger",
    "UiPathResumeTriggerType",
    "UiPathApiTrigger",
    "UiPathResumeTriggerName",
]

from uipath.core.triggers.trigger import (
    UiPathApiTrigger,
    UiPathResumeTrigger,
    UiPathResumeTriggerName,
    UiPathResumeTriggerType,
)



"""Serialization utilities for converting Python objects to various formats."""

from .json import serialize_defaults, serialize_json, serialize_object

__all__ = ["serialize_defaults", "serialize_json", "serialize_object"]



"""UiPath common exceptions."""

from enum import Enum


class ErrorCategory(str, Enum):
    """Categories of UiPath errors."""

    DEPLOYMENT = "Deployment"
    SYSTEM = "System"
    UNKNOWN = "Unknown"
    USER = "User"


class UiPathFaultedTriggerError(Exception):
    """UiPath resume trigger error."""

    category: ErrorCategory
    message: str
    detail: str

    def __init__(self, category: ErrorCategory, message: str, detail: str = ""):
        """Initialize the UiPathFaultedTriggerError."""
        self.category = category
        self.message = message
        self.detail = detail
        super().__init__(f"{message}: {detail}" if detail else message)


class UiPathPendingTriggerError(UiPathFaultedTriggerError):
    """Custom resume trigger error for pending triggers."""

    pass
